@@ -20,10 +20,9 @@ Main window for Bitmask.
 import logging
 import os
 
-import keyring
-
 from PySide import QtCore, QtGui
 from twisted.internet import threads
+from zope.proxy import ProxyBase, setProxiedObject, sameProxiedObjects
 
 from leap.bitmask import __version__ as VERSION
 from leap.bitmask.config.leapsettings import LeapSettings
@@ -39,18 +38,15 @@ from leap.bitmask.gui.mail_status import MailStatusWidget
 from leap.bitmask.gui.wizard import Wizard
 
 from leap.bitmask import provider
-from leap.bitmask.provider.providerbootstrapper import ProviderBootstrapper
-from leap.bitmask.services.eip.eipbootstrapper import EIPBootstrapper
-from leap.bitmask.services.eip import eipconfig
-# XXX: Soledad might not work out of the box in Windows, issue #2932
-from leap.bitmask.services.soledad.soledadbootstrapper import \
-    SoledadBootstrapper
-from leap.bitmask.services.mail.smtpbootstrapper import SMTPBootstrapper
-from leap.bitmask.services.mail import imap
 from leap.bitmask.platform_init import IS_WIN, IS_MAC
 from leap.bitmask.platform_init.initializers import init_platform
+from leap.bitmask.provider.providerbootstrapper import ProviderBootstrapper
 
+from leap.bitmask.services.mail import conductor as mail_conductor
+
+from leap.bitmask.services.eip import eipconfig
 from leap.bitmask.services.eip import get_openvpn_management
+from leap.bitmask.services.eip.eipbootstrapper import EIPBootstrapper
 from leap.bitmask.services.eip.connection import EIPConnection
 from leap.bitmask.services.eip.vpnprocess import VPN
 from leap.bitmask.services.eip.vpnprocess import OpenVPNAlreadyRunning
@@ -62,12 +58,12 @@ from leap.bitmask.services.eip.linuxvpnlauncher import EIPNoPkexecAvailable
 from leap.bitmask.services.eip.linuxvpnlauncher import \
     EIPNoPolkitAuthAgentAvailable
 from leap.bitmask.services.eip.darwinvpnlauncher import EIPNoTunKextLoaded
+from leap.bitmask.services.soledad.soledadbootstrapper import \
+    SoledadBootstrapper
 
 from leap.bitmask.util.keyring_helpers import has_keyring
 from leap.bitmask.util.leap_log_handler import LeapLogHandler
 
-from leap.bitmask.services.mail.smtpconfig import SMTPConfig
-
 if IS_WIN:
     from leap.bitmask.platform_init.locks import WindowsLock
     from leap.bitmask.platform_init.locks import raise_window_ack
@@ -90,13 +86,6 @@ class MainWindow(QtGui.QMainWindow):
     LOGIN_INDEX = 0
     EIP_STATUS_INDEX = 1
 
-    # Keyring
-    KEYRING_KEY = "bitmask"
-
-    # SMTP
-    PORT_KEY = "port"
-    IP_KEY = "ip_address"
-
     OPENVPN_SERVICE = "openvpn"
     MX_SERVICE = "mx"
 
@@ -174,10 +163,12 @@ class MainWindow(QtGui.QMainWindow):
 
         self._eip_connection = EIPConnection()
 
+        # XXX this should be handled by EIP Conductor
         self._eip_connection.qtsigs.connecting_signal.connect(
             self._start_eip)
         self._eip_connection.qtsigs.disconnecting_signal.connect(
             self._stop_eip)
+
         self._eip_status.eip_connection_connected.connect(
             self._on_eip_connected)
         self.eip_needs_login.connect(
@@ -228,13 +219,22 @@ class MainWindow(QtGui.QMainWindow):
             self._eip_intermediate_stage)
         self._eip_bootstrapper.download_client_certificate.connect(
             self._finish_eip_bootstrap)
+
         self._vpn = VPN(openvpn_verb=openvpn_verb)
+
+        # connect vpn process signals
         self._vpn.qtsigs.state_changed.connect(
             self._eip_status.update_vpn_state)
         self._vpn.qtsigs.status_changed.connect(
             self._eip_status.update_vpn_status)
         self._vpn.qtsigs.process_finished.connect(
             self._eip_finished)
+        self._vpn.qtsigs.network_unreachable.connect(
+            self._on_eip_network_unreachable)
+        self._vpn.qtsigs.process_restart_tls.connect(
+            self._do_eip_restart)
+        self._vpn.qtsigs.process_restart_ping.connect(
+            self._do_eip_restart)
 
         self._soledad_bootstrapper = SoledadBootstrapper()
         self._soledad_bootstrapper.download_config.connect(
@@ -243,12 +243,8 @@ class MainWindow(QtGui.QMainWindow):
             self._soledad_bootstrapped_stage)
         self._soledad_bootstrapper.soledad_timeout.connect(
             self._retry_soledad_connection)
-        # XXX missing connect to soledad_failed (signal unrecoverable to user)
-        # TODO wait until chiiph ui refactor.
-
-        self._smtp_bootstrapper = SMTPBootstrapper()
-        self._smtp_bootstrapper.download_config.connect(
-            self._smtp_bootstrapped_stage)
+        self._soledad_bootstrapper.soledad_failed.connect(
+            self._mail_status.set_soledad_failed)
 
         self.ui.action_about_leap.triggered.connect(self._about)
         self.ui.action_quit.triggered.connect(self.quit)
@@ -258,7 +254,7 @@ class MainWindow(QtGui.QMainWindow):
             self._launch_wizard)
 
         if IS_MAC:
-            self.ui.menuFile.menuAction().setText(self.tr("Util"))
+            self.ui.menuFile.menuAction().setText(self.tr("File"))
 
         self.raise_window.connect(self._do_raise_mainwindow)
 
@@ -267,8 +263,9 @@ class MainWindow(QtGui.QMainWindow):
 
         self._systray = None
 
+        # XXX separate actions into a different
+        # module.
         self._action_mail_status = QtGui.QAction(self.tr("Mail is OFF"), self)
-        self._action_mail_status.setEnabled(False)
         self._mail_status.set_action_mail_status(self._action_mail_status)
 
         self._action_eip_startstop = QtGui.QAction("", self)
@@ -286,16 +283,22 @@ class MainWindow(QtGui.QMainWindow):
 
         self.ui.lblNewUpdates.setVisible(False)
         self.ui.btnMore.setVisible(False)
+        #########################################
+        # We hide this in height temporarily too
+        self.ui.lblNewUpdates.resize(0, 0)
+        self.ui.btnMore.resize(0, 0)
+        #########################################
         self.ui.btnMore.clicked.connect(self._updates_details)
 
         # Services signals/slots connection
         self.new_updates.connect(self._react_to_new_updates)
+
+        # XXX should connect to mail_conductor.start_mail_service instead
+        self.soledad_ready.connect(self._start_smtp_bootstrapping)
         self.soledad_ready.connect(self._start_imap_service)
-        self.soledad_ready.connect(self._set_soledad_ready)
         self.mail_client_logged_in.connect(self._fetch_incoming_mail)
         self.logout.connect(self._stop_imap_service)
         self.logout.connect(self._stop_smtp_service)
-        self.logout.connect(self._mail_status.stopped_mail)
 
         ################################# end Qt Signals connection ########
 
@@ -308,17 +311,18 @@ class MainWindow(QtGui.QMainWindow):
 
         self._bypass_checks = bypass_checks
 
-        self._soledad = None
-        self._soledad_ready = False
-        self._keymanager = None
-        self._smtp_service = None
-        self._smtp_port = None
-        self._imap_service = None
+        # We initialize Soledad and Keymanager instances as
+        # transparent proxies, so we can pass the reference freely
+        # around.
+        self._soledad = ProxyBase(None)
+        self._keymanager = ProxyBase(None)
 
         self._login_defer = None
         self._download_provider_defer = None
 
-        self._smtp_config = SMTPConfig()
+        self._mail_conductor = mail_conductor.MailConductor(
+            self._soledad, self._keymanager)
+        self._mail_conductor.connect_mail_signals(self._mail_status)
 
         # Eip machine is a public attribute where the state machine for
         # the eip connection will be available to the different components.
@@ -330,6 +334,7 @@ class MainWindow(QtGui.QMainWindow):
         self.eip_machine = None
         # start event machines
         self.start_eip_machine()
+        self._mail_conductor.start_mail_machine(parent=self)
 
         if self._first_run():
             self._wizard_firstrun = True
@@ -352,8 +357,12 @@ class MainWindow(QtGui.QMainWindow):
         finishing.
         """
         if self._wizard_firstrun:
-            self._settings.set_properprovider(False)
-            self.quit()
+            providers = self._settings.get_configured_providers()
+            has_provider_on_disk = len(providers) != 0
+            if not has_provider_on_disk:
+                # if we don't have any provider configured (included a pinned
+                # one) we can't use the application, so quit.
+                self.quit()
         else:
             self._finish_init()
 
@@ -381,6 +390,7 @@ class MainWindow(QtGui.QMainWindow):
         if IS_MAC:
             self._wizard.raise_()
         self._wizard.finished.connect(self._wizard_finished)
+        self._settings.set_skip_first_run(True)
 
     def _wizard_finished(self):
         """
@@ -399,6 +409,8 @@ class MainWindow(QtGui.QMainWindow):
         :return: a logging handler or None
         :rtype: LeapLogHandler or None
         """
+        # TODO this can be a function, does not need
+        # to be a method.
         leap_logger = logging.getLogger('leap')
         for h in leap_logger.handlers:
             if isinstance(h, LeapLogHandler):
@@ -434,13 +446,11 @@ class MainWindow(QtGui.QMainWindow):
 
         Displays the preferences window.
         """
-        preferences_window = PreferencesWindow(self, self._srp_auth)
+        preferences_window = PreferencesWindow(self, self._srp_auth,
+                                               self._provider_config)
 
-        if self._soledad_ready:
-            preferences_window.set_soledad_ready(self._soledad)
-        else:
-            self.soledad_ready.connect(
-                lambda: preferences_window.set_soledad_ready(self._soledad))
+        self.soledad_ready.connect(
+            lambda: preferences_window.set_soledad_ready(self._soledad))
 
         preferences_window.show()
 
@@ -454,15 +464,9 @@ class MainWindow(QtGui.QMainWindow):
         """
         EIPPreferencesWindow(self).show()
 
-    def _set_soledad_ready(self):
-        """
-        SLOT
-        TRIGGERS:
-            self.soledad_ready
-
-        It sets the soledad object as ready to use.
-        """
-        self._soledad_ready = True
+    #
+    # updates
+    #
 
     def _new_updates_available(self, req):
         """
@@ -534,6 +538,8 @@ class MainWindow(QtGui.QMainWindow):
         if IS_MAC:
             self.raise_()
 
+        self._hide_unsupported_services()
+
         if self._wizard:
             possible_username = self._wizard.get_username()
             possible_password = self._wizard.get_password()
@@ -553,81 +559,50 @@ class MainWindow(QtGui.QMainWindow):
                 self._login_widget.set_password(possible_password)
                 self._login()
             self._wizard = None
-            self._settings.set_properprovider(True)
         else:
             self._try_autostart_eip()
+
+            domain = self._settings.get_provider()
+            if domain is not None:
+                self._login_widget.select_provider_by_name(domain)
+
             if not self._settings.get_remember():
                 # nothing to do here
                 return
 
             saved_user = self._settings.get_user()
 
-            try:
-                username, domain = saved_user.split('@')
-            except (ValueError, AttributeError) as e:
-                # if the saved_user does not contain an '@' or its None
-                logger.error('Username@provider malformed. %r' % (e, ))
-                saved_user = None
-
             if saved_user is not None and has_keyring():
-                # fill the username
-                self._login_widget.set_user(username)
-
-                # select the configured provider in the combo box
-                self._login_widget.select_provider_by_name(domain)
-
-                self._login_widget.set_remember(True)
-
-                saved_password = None
-                try:
-                    saved_password = keyring.get_password(self.KEYRING_KEY,
-                                                          saved_user
-                                                          .encode("utf8"))
-                except ValueError, e:
-                    logger.debug("Incorrect Password. %r." % (e,))
-
-                if saved_password is not None:
-                    self._login_widget.set_password(
-                        saved_password.decode("utf8"))
+                if self._login_widget.load_user_from_keyring(saved_user):
                     self._login()
 
-    def _try_autostart_eip(self):
+    def _hide_unsupported_services(self):
         """
-        Tries to autostart EIP
+        Given a set of configured providers, it creates a set of
+        available services among all of them and displays the service
+        widgets of only those.
+
+        This means, for example, that with just one provider with EIP
+        only, the mail widget won't be displayed.
         """
-        settings = self._settings
+        providers = self._settings.get_configured_providers()
 
-        should_autostart = settings.get_autostart_eip()
-        if not should_autostart:
-            logger.debug('Will not autostart EIP since it is setup '
-                         'to not to do it')
-            self.eip_needs_login.emit()
-            return
+        services = set()
 
-        default_provider = settings.get_defaultprovider()
+        for prov in providers:
+            provider_config = ProviderConfig()
+            loaded = provider_config.load(
+                provider.get_provider_path(prov))
+            if loaded:
+                for service in provider_config.get_services():
+                    services.add(service)
 
-        if default_provider is None:
-            logger.info("Cannot autostart Encrypted Internet because there is "
-                        "no default provider configured")
-            self.eip_needs_login.emit()
-            return
+        self.ui.eipWidget.setVisible(self.OPENVPN_SERVICE in services)
+        self.ui.mailWidget.setVisible(self.MX_SERVICE in services)
 
-        self._enabled_services = settings.get_enabled_services(
-            default_provider)
-
-        loaded = self._provisional_provider_config.load(
-            provider.get_provider_path(default_provider))
-        if loaded:
-            # XXX I think we should not try to re-download config every time,
-            # it adds some delay.
-            # Maybe if it's the first run in a session,
-            # or we can try only if it fails.
-            self._download_eip_config()
-        else:
-            # XXX: Display a proper message to the user
-            self.eip_needs_login.emit()
-            logger.error("Unable to load %s config, cannot autostart." %
-                         (default_provider,))
+    #
+    # systray
+    #
 
     def _show_systray(self):
         """
@@ -649,7 +624,7 @@ class MainWindow(QtGui.QMainWindow):
         eip_menu = systrayMenu.addMenu(self.tr("Encrypted Internet: OFF"))
         eip_menu.addAction(self._action_eip_startstop)
         self._eip_status.set_eip_status_menu(eip_menu)
-
+        systrayMenu.addSeparator()
         systrayMenu.addAction(self._action_mail_status)
         systrayMenu.addSeparator()
         systrayMenu.addAction(self.ui.action_quit)
@@ -799,8 +774,8 @@ class MainWindow(QtGui.QMainWindow):
         """
         providers = self._settings.get_configured_providers()
         has_provider_on_disk = len(providers) != 0
-        is_proper_provider = self._settings.get_properprovider()
-        return not (has_provider_on_disk and is_proper_provider)
+        skip_first_run = self._settings.get_skip_first_run()
+        return not (has_provider_on_disk and skip_first_run)
 
     def _download_provider_config(self):
         """
@@ -808,6 +783,7 @@ class MainWindow(QtGui.QMainWindow):
         provider configuration if it's not present, otherwise will
         emit the corresponding signals inmediately
         """
+        # XXX should rename this provider, name clash.
         provider = self._login_widget.get_selected_provider()
 
         pb = self._provider_bootstrapper
@@ -828,6 +804,7 @@ class MainWindow(QtGui.QMainWindow):
         :type data: dict
         """
         if data[self._provider_bootstrapper.PASSED_KEY]:
+            # XXX should rename this provider, name clash.
             provider = self._login_widget.get_selected_provider()
 
             # If there's no loaded provider or
@@ -900,8 +877,10 @@ class MainWindow(QtGui.QMainWindow):
         leap_assert(self._provider_config, "We need a provider config!")
 
         if data[self._provider_bootstrapper.PASSED_KEY]:
-            username = self._login_widget.get_user().encode("utf8")
-            password = self._login_widget.get_password().encode("utf8")
+            username = self._login_widget.get_user()
+            password = self._login_widget.get_password()
+
+            self._hide_unsupported_services()
 
             if self._srp_auth is None:
                 self._srp_auth = SRPAuth(self._provider_config)
@@ -951,6 +930,7 @@ class MainWindow(QtGui.QMainWindow):
         """
 
         self._login_widget.logged_in()
+        self.ui.lblLoginProvider.setText(self._provider_config.get_name())
 
         self._enabled_services = self._settings.get_enabled_services(
             self._provider_config.get_domain())
@@ -971,7 +951,11 @@ class MainWindow(QtGui.QMainWindow):
 
         self._download_eip_config()
 
+    ###################################################################
+    # Service control methods: soledad
+
     def _soledad_intermediate_stage(self, data):
+        # TODO missing param docstring
         """
         SLOT
         TRIGGERS:
@@ -1023,114 +1007,58 @@ class MainWindow(QtGui.QMainWindow):
             logger.debug("ERROR on soledad bootstrapping:")
             logger.error("%r" % data[self._soledad_bootstrapper.ERROR_KEY])
             return
-        else:
-            logger.debug("Done bootstrapping Soledad")
 
-            self._soledad = self._soledad_bootstrapper.soledad
-            self._keymanager = self._soledad_bootstrapper.keymanager
+        logger.debug("Done bootstrapping Soledad")
+
+        # Update the proxy objects to point to
+        # the initialized instances.
+        setProxiedObject(self._soledad,
+                         self._soledad_bootstrapper.soledad)
+        setProxiedObject(self._keymanager,
+                         self._soledad_bootstrapper.keymanager)
 
         # Ok, now soledad is ready, so we can allow other things that
         # depend on soledad to start.
 
         # this will trigger start_imap_service
+        # and start_smtp_boostrapping
         self.soledad_ready.emit()
 
-        # TODO connect all these activations to the soledad_ready
-        # signal so the logic is clearer to follow.
-
-        if self._provider_config.provides_mx() and \
-                self._enabled_services.count(self.MX_SERVICE) > 0:
-            self._smtp_bootstrapper.run_smtp_setup_checks(
-                self._provider_config,
-                self._smtp_config,
-                True)
-
     ###################################################################
     # Service control methods: smtp
 
-    def _smtp_bootstrapped_stage(self, data):
+    @QtCore.Slot()
+    def _start_smtp_bootstrapping(self):
         """
         SLOT
         TRIGGERS:
-          self._smtp_bootstrapper.download_config
-
-        If there was a problem, displays it, otherwise it does nothing.
-        This is used for intermediate bootstrapping stages, in case
-        they fail.
-
-        :param data: result from the bootstrapping stage for Soledad
-        :type data: dict
+            self.soledad_ready
         """
-        passed = data[self._smtp_bootstrapper.PASSED_KEY]
-        if not passed:
-            logger.error(data[self._smtp_bootstrapper.ERROR_KEY])
-            return
-        logger.debug("Done bootstrapping SMTP")
-        self._check_smtp_config()
-
-    def _check_smtp_config(self):
-        """
-        Checks smtp config and tries to download smtp client cert if needed.
-        """
-        hosts = self._smtp_config.get_hosts()
-        # TODO handle more than one host and define how to choose
-        if len(hosts) > 0:
-            hostname = hosts.keys()[0]
-            logger.debug("Using hostname %s for SMTP" % (hostname,))
-            host = hosts[hostname][self.IP_KEY].encode("utf-8")
-            port = hosts[hostname][self.PORT_KEY]
-
-            client_cert = self._smtp_config.get_client_cert_path(
+        # TODO for simmetry, this should be called start_smtp_service
+        # (and delegate all the checks to the conductor)
+        if self._provider_config.provides_mx() and \
+                self._enabled_services.count(self.MX_SERVICE) > 0:
+            self._mail_conductor.smtp_bootstrapper.run_smtp_setup_checks(
                 self._provider_config,
-                about_to_download=True)
-
-            if not os.path.isfile(client_cert):
-                self._smtp_bootstrapper._download_client_certificates()
-            if os.path.isfile(client_cert):
-                self._start_smtp_service(host, port, client_cert)
-            else:
-                logger.warning("Tried to download email client "
-                               "certificate, but could not find any")
-
-        else:
-            logger.warning("No smtp hosts configured")
-
-    def _start_smtp_service(self, host, port, cert):
-        """
-        Starts the smtp service.
-        """
-        # TODO Make the encrypted_only configurable
-        # TODO pick local smtp port in a better way
-        # TODO remove hard-coded port and let leap.mail set
-        # the specific default.
-
-        from leap.mail.smtp import setup_smtp_relay
-        self._smtp_service, self._smtp_port = setup_smtp_relay(
-            port=2013,
-            keymanager=self._keymanager,
-            smtp_host=host,
-            smtp_port=port,
-            smtp_cert=cert,
-            smtp_key=cert,
-            encrypted_only=False)
+                self._mail_conductor.smtp_config,
+                download_if_needed=True)
 
+    # XXX --- should remove from here, and connecte directly to the state
+    # machine.
+    @QtCore.Slot()
     def _stop_smtp_service(self):
         """
         SLOT
         TRIGGERS:
             self.logout
         """
-        # There is a subtle difference here:
-        # we are stopping the factory for the smtp service here,
-        # but in the imap case we are just stopping the fetcher.
-        if self._smtp_service is not None:
-            logger.debug('Stopping smtp service.')
-            self._smtp_port.stopListening()
-            self._smtp_service.doStop()
+        # TODO call stop_mail_service
+        self._mail_conductor.stop_smtp_service()
 
     ###################################################################
     # Service control methods: imap
 
+    @QtCore.Slot()
     def _start_imap_service(self):
         """
         SLOT
@@ -1139,11 +1067,7 @@ class MainWindow(QtGui.QMainWindow):
         """
         if self._provider_config.provides_mx() and \
                 self._enabled_services.count(self.MX_SERVICE) > 0:
-            logger.debug('Starting imap service')
-
-            self._imap_service = imap.start_imap_service(
-                self._soledad,
-                self._keymanager)
+            self._mail_conductor.start_imap_service()
 
     def _on_mail_client_logged_in(self, req):
         """
@@ -1151,30 +1075,25 @@ class MainWindow(QtGui.QMainWindow):
         """
         self.mail_client_logged_in.emit()
 
+    @QtCore.Slot()
     def _fetch_incoming_mail(self):
         """
         SLOT
         TRIGGERS:
             self.mail_client_logged_in
         """
-        # TODO have a mutex over fetch operation.
-        if self._imap_service:
-            logger.debug('Client connected, fetching mail...')
-            self._imap_service.fetch()
+        # TODO connect signal directly!!!
+        self._mail_conductor.fetch_incoming_mail()
 
+    @QtCore.Slot()
     def _stop_imap_service(self):
         """
         SLOT
         TRIGGERS:
             self.logout
         """
-        # There is a subtle difference here:
-        # we are just stopping the fetcher here,
-        # but in the smtp case we are stopping the factory.
-        # We should homogenize both services.
-        if self._imap_service is not None:
-            logger.debug('Stopping imap service.')
-            self._imap_service.stop()
+        # TODO call stop_mail_service
+        self._mail_conductor.stop_imap_service()
 
     # end service control methods (imap)
 
@@ -1222,16 +1141,53 @@ class MainWindow(QtGui.QMainWindow):
         signal that currently is beeing processed under status_panel.
         After the refactor to EIPConductor this should not be necessary.
         """
-        logger.debug('EIP connected signal received ...')
         self._eip_connection.qtsigs.connected_signal.emit()
 
+    def _try_autostart_eip(self):
+        """
+        Tries to autostart EIP
+        """
+        settings = self._settings
+
+        should_autostart = settings.get_autostart_eip()
+        if not should_autostart:
+            logger.debug('Will not autostart EIP since it is setup '
+                         'to not to do it')
+            self.eip_needs_login.emit()
+            return
+
+        default_provider = settings.get_defaultprovider()
+
+        if default_provider is None:
+            logger.info("Cannot autostart Encrypted Internet because there is "
+                        "no default provider configured")
+            self.eip_needs_login.emit()
+            return
+
+        self._enabled_services = settings.get_enabled_services(
+            default_provider)
+
+        loaded = self._provisional_provider_config.load(
+            provider.get_provider_path(default_provider))
+        if loaded:
+            # XXX I think we should not try to re-download config every time,
+            # it adds some delay.
+            # Maybe if it's the first run in a session,
+            # or we can try only if it fails.
+            self._download_eip_config()
+        else:
+            # XXX: Display a proper message to the user
+            self.eip_needs_login.emit()
+            logger.error("Unable to load %s config, cannot autostart." %
+                         (default_provider,))
+
     @QtCore.Slot()
     def _start_eip(self):
         """
         SLOT
         TRIGGERS:
-          self._eip_status.start_eip
-          self._action_eip_startstop.triggered
+          self._eip_connection.qtsigs.do_connect_signal
+          (via state machine)
         or called from _finish_eip_bootstrap
 
         Starts EIP
@@ -1332,8 +1288,8 @@ class MainWindow(QtGui.QMainWindow):
         """
         SLOT
         TRIGGERS:
-          self._eip_status.stop_eip
-          self._action_eip_startstop.triggered
+          self._eip_connection.qtsigs.do_disconnect_signal
+          (via state machine)
         or called from _eip_finished
 
         Stops vpn process and makes gui adjustments to reflect
@@ -1357,14 +1313,100 @@ class MainWindow(QtGui.QMainWindow):
                            self._get_best_provider_config().get_domain()))
         self._eip_status.eip_stopped()
 
+    @QtCore.Slot()
+    def _on_eip_network_unreachable(self):
+        # XXX Should move to EIP Conductor
+        """
+        SLOT
+        TRIGGERS:
+            self._eip_connection.qtsigs.network_unreachable
+
+        Displays a "network unreachable" error in the EIP status panel.
+        """
+        self._eip_status.set_eip_status(self.tr("Network is unreachable"),
+                                        error=True)
+        self._eip_status.set_eip_status_icon("error")
+
+    @QtCore.Slot()
+    def _do_eip_restart(self):
+        # XXX Should move to EIP Conductor
+        """
+        SLOT
+            self._eip_connection.qtsigs.process_restart
+
+        Restart the connection.
+        """
+        # for some reason, emitting the do_disconnect/do_connect
+        # signals hangs the UI.
+        self._stop_eip()
+        QtCore.QTimer.singleShot(2000, self._start_eip)
+
     def _set_eipstatus_off(self, error=True):
         """
         Sets eip status to off
         """
+        # XXX this should be handled by the state machine.
         self._eip_status.set_eip_status(self.tr("EIP has stopped"),
                                         error=error)
         self._eip_status.set_eip_status_icon("error")
 
+    def _eip_finished(self, exitCode):
+        """
+        SLOT
+        TRIGGERS:
+          self._vpn.process_finished
+
+        Triggered when the EIP/VPN process finishes to set the UI
+        accordingly.
+
+        Ideally we would have the right exit code here,
+        but the use of different wrappers (pkexec, cocoasudo) swallows
+        the openvpn exit code so we get zero exit in some cases  where we
+        shouldn't. As a workaround we just use a flag to indicate
+        a purposeful switch off, and mark everything else as unexpected.
+
+        In the near future we should trigger a native notification from here,
+        since the user really really wants to know she is unprotected asap.
+        And the right thing to do will be to fail-close.
+
+        :param exitCode: the exit code of the eip process.
+        :type exitCode: int
+        """
+        # TODO move to EIPConductor.
+        # TODO Add error catching to the openvpn log observer
+        # so we can have a more precise idea of which type
+        # of error did we have (server side, local problem, etc)
+
+        logger.info("VPN process finished with exitCode %s..."
+                    % (exitCode,))
+
+        qtsigs = self._eip_connection.qtsigs
+        signal = qtsigs.disconnected_signal
+
+        # XXX check if these exitCodes are pkexec/cocoasudo specific
+        if exitCode in (126, 127):
+            self._eip_status.set_eip_status(
+                self.tr("Encrypted Internet could not be launched "
+                        "because you did not authenticate properly."),
+                error=True)
+            self._vpn.killit()
+            signal = qtsigs.connection_aborted_signal
+
+        elif exitCode != 0 or not self.user_stopped_eip:
+            self._eip_status.set_eip_status(
+                self.tr("Encrypted Internet finished in an "
+                        "unexpected manner!"), error=True)
+            signal = qtsigs.connection_died_signal
+
+        if exitCode == 0 and IS_MAC:
+            # XXX remove this warning after I fix cocoasudo.
+            logger.warning("The above exit code MIGHT BE WRONG.")
+
+        # We emit signals to trigger transitions in the state machine:
+        signal.emit()
+
+    # eip boostrapping, config etc...
+
     def _download_eip_config(self):
         """
         Starts the EIP bootstrapping sequence
@@ -1426,7 +1468,25 @@ class MainWindow(QtGui.QMainWindow):
                         "Configuration."),
                 error=True)
 
-    # end eip methods -------------------------------------------
+    def _eip_intermediate_stage(self, data):
+        # TODO missing param
+        """
+        SLOT
+        TRIGGERS:
+          self._eip_bootstrapper.download_config
+
+        If there was a problem, displays it, otherwise it does nothing.
+        This is used for intermediate bootstrapping stages, in case
+        they fail.
+        """
+        passed = data[self._provider_bootstrapper.PASSED_KEY]
+        if not passed:
+            self._login_widget.set_status(
+                self.tr("Unable to connect: Problem with provider"))
+            logger.error(data[self._provider_bootstrapper.ERROR_KEY])
+            self._already_started_eip = False
+
+    # end of EIP methods ---------------------------------------------
 
     def _get_best_provider_config(self):
         """
@@ -1469,6 +1529,7 @@ class MainWindow(QtGui.QMainWindow):
         self.logout.emit()
 
     def _done_logging_out(self, ok, message):
+        # TODO missing params in docstring
         """
         SLOT
         TRIGGER: self._srp_auth.logout_finished
@@ -1477,11 +1538,12 @@ class MainWindow(QtGui.QMainWindow):
         logging out
         """
         self._login_widget.done_logout()
+        self.ui.lblLoginProvider.setText(self.tr("Login"))
 
         if ok:
             self._logged_user = None
-
             self._login_widget.logged_out()
+            self._mail_status.mail_state_disabled()
 
         else:
             self._login_widget.set_login_status(
@@ -1489,6 +1551,7 @@ class MainWindow(QtGui.QMainWindow):
                 error=True)
 
     def _intermediate_stage(self, data):
+        # TODO this method name is confusing as hell.
         """
         SLOT
         TRIGGERS:
@@ -1508,80 +1571,9 @@ class MainWindow(QtGui.QMainWindow):
                 self.tr("Unable to connect: Problem with provider"))
             logger.error(data[self._provider_bootstrapper.ERROR_KEY])
 
-    def _eip_intermediate_stage(self, data):
-        """
-        SLOT
-        TRIGGERS:
-          self._eip_bootstrapper.download_config
-
-        If there was a problem, displays it, otherwise it does nothing.
-        This is used for intermediate bootstrapping stages, in case
-        they fail.
-        """
-        passed = data[self._provider_bootstrapper.PASSED_KEY]
-        if not passed:
-            self._login_widget.set_status(
-                self.tr("Unable to connect: Problem with provider"))
-            logger.error(data[self._provider_bootstrapper.ERROR_KEY])
-            self._already_started_eip = False
-
-    def _eip_finished(self, exitCode):
-        """
-        SLOT
-        TRIGGERS:
-          self._vpn.process_finished
-
-        Triggered when the EIP/VPN process finishes to set the UI
-        accordingly.
-        """
-        # TODO move to EIPConductor.
-        logger.info("VPN process finished with exitCode %s..."
-                    % (exitCode,))
-
-        # Ideally we would have the right exit code here,
-        # but the use of different wrappers (pkexec, cocoasudo) swallows
-        # the openvpn exit code so we get zero exit in some cases  where we
-        # shouldn't. As a workaround we just use a flag to indicate
-        # a purposeful switch off, and mark everything else as unexpected.
-
-        # In the near future we should trigger a native notification from here,
-        # since the user really really wants to know she is unprotected asap.
-        # And the right thing to do will be to fail-close.
-
-        # TODO we should have a way of parsing the latest lines in the vpn
-        # log buffer so we can have a more precise idea of which type
-        # of error did we have (server side, local problem, etc)
-
-        qtsigs = self._eip_connection.qtsigs
-        signal = qtsigs.disconnected_signal
-
-        # XXX check if these exitCodes are pkexec/cocoasudo specific
-        if exitCode in (126, 127):
-            self._eip_status.set_eip_status(
-                self.tr("Encrypted Internet could not be launched "
-                        "because you did not authenticate properly."),
-                error=True)
-            self._vpn.killit()
-            signal = qtsigs.connection_aborted_signal
-
-        elif exitCode != 0 or not self.user_stopped_eip:
-            self._eip_status.set_eip_status(
-                self.tr("Encrypted Internet finished in an "
-                        "unexpected manner!"), error=True)
-            signal = qtsigs.connection_died_signal
-
-        if exitCode == 0 and IS_MAC:
-            # XXX remove this warning after I fix cocoasudo.
-            logger.warning("The above exit code MIGHT BE WRONG.")
-
-        # XXX verify that the logic kees the same w/o the abnormal flag
-        # after the refactor to EIPConnection has been completed
-        # (eipconductor taking the most of the logic under transitions
-        # that right now are handled under status_panel)
-        #self._stop_eip(abnormal)
-
-        # We emit signals to trigger transitions in the state machine:
-        signal.emit()
+    #
+    # window handling methods
+    #
 
     def _on_raise_window_event(self, req):
         """
@@ -1607,6 +1599,10 @@ class MainWindow(QtGui.QMainWindow):
         if IS_MAC:
             self.raise_()
 
+    #
+    # cleanup and quit methods
+    #
+
     def _cleanup_pidfiles(self):
         """
         Removes lockfiles on a clean shutdown.
@@ -1623,8 +1619,7 @@ class MainWindow(QtGui.QMainWindow):
         """
         logger.debug('About to quit, doing cleanup...')
 
-        if self._imap_service is not None:
-            self._imap_service.stop()
+        self._mail_conductor.stop_imap_service()
 
         if self._srp_auth is not None:
             if self._srp_auth.get_session_id() is not None or \
